/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebMail Notifier.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsIWebMailNotifier = Components.interfaces.nsIWebMailNotifier;

// UUID uniquely identifying our component
const CLASS_ID = Components.ID("{e3ab0980-b4e1-11db-abbd-0800200c9a66}");

// description
const CLASS_NAME = "WebMail Notifier Service";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/WebMailNotifier;1";

const ST_PRE         = 0;
const ST_PRE_RES     = 1;
const ST_LOGIN       = 10;
const ST_LOGIN_RES   = 11;
const ST_DATA        = 20;
const ST_DATA_RES    = 21;

const LOGIN_URL      = 0;
const LOGIN_ID       = 1;
const LOGIN_PW       = 2;
const LOGIN_DATA     = 3;
const LOGIN_POST     = 1;

const dout=Components.utils.reportError;
var enableDebug=false;
/***********************************************************
class definition
***********************************************************/

//class constructor
function WebMailNotifier() {
  //preference observer
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  this.prefBranch = prefService.getBranch("wm-notifier.");
  this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.prefBranch.addObserver("", this, false);
  this.timerDelay=this.prefBranch.getIntPref("updateInterval")*60000;
  this.newMailsOnly=this.prefBranch.getBoolPref("newMailsOnly");
  enableDebug=this.prefBranch.getIntPref("debug");
  var dir = Components.stack.toString();
  dir=dir.match(/:: (.+\/).+?/);
  dir=dir[1];
  var hosts=this.prefBranch.getIntPref("hosts");
  var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                  .getService(Components.interfaces.mozIJSSubScriptLoader);
  for(var i=0;i<hosts;i++){
    try{
      loader.loadSubScript(dir+"webmail"+i+".js");
     }catch(e){
dout(e);
     }
  }
  this.initHandlers(false);
  var observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);
  observerService.addObserver(this, "http-on-modify-request", false);
  observerService.addObserver(this, "http-on-examine-response", false);
}

// class definition
WebMailNotifier.prototype = {
  // define the function we want to expose in our interface
  listeners : new Array(),
  timer : null,
  countEnabled : 0,
  countChecked : 0,
  countTotal : 0,
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsIWebMailNotifier) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
}

WebMailNotifier.prototype.initHandlers = function(aCheckNow) {
  this.webMailHandlers=[];
  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                .getService(Components.interfaces.nsIPasswordManager);
  var e = passwordManager.enumerator;
  var str="chrome://wm-notifier/webmail";
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
  while (e.hasMoreElements()) {
    try {
      var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
      if (pass.host.indexOf(str)==0) {
        var id=pass.host.substring(str.length);
        var obj=new WebMailHandler(this);
        this.webMailHandlers.push(obj);
        obj.id=id;
        obj.user=pass.user;
        obj.password=pass.password;
        eval("initHandler"+id+"(this.webMailHandlers["+(this.webMailHandlers.length-1)+"])");
        obj.dataURI=ioService.newURI(obj.dataURL,null,null);
        var post=(obj.loginData[LOGIN_DATA]?obj.loginData[LOGIN_DATA]:"")
              +(obj.loginData[LOGIN_ID]?("&"+obj.loginData[LOGIN_ID]+"="+encodeURIComponent(obj.user)):"")
              +(obj.loginData[LOGIN_PW]?("&"+obj.loginData[LOGIN_PW]+"="+encodeURIComponent(obj.password)):"");
        obj.loginData.splice(1,obj.loginData.length-1);
        obj.loginData[LOGIN_POST]=post;
        obj.stage=obj.initStage;
      }
    }catch(ex) {
    }
  }
  this.webMailHandlers.sort(this.sortFunc);
  this.buildTable(aCheckNow);
}
WebMailNotifier.prototype.sortFunc = function(a,b) {
  if(a.id<b.id)return -1;
  else if(a.id==b.id){
    if(a.user<b.user)return -1;
    else if(a.user==b.user)return 0;
    else return 1;
  }else return 1;
}
WebMailNotifier.prototype.buildTable = function(aCheckNow) {
  this.hdlTable=[];
  var defaults=[];
  var hosts=this.prefBranch.getIntPref("hosts");
  for(var i=0;i<hosts;i++){
    this.hdlTable.push([]);
    try{
        defaults[i]=this.prefBranch.getCharPref("webmail"+i+".default");
    }catch(e){}
  }
  this.countEnabled=0;
  for each(var o in this.webMailHandlers){
    try{
      o.enabled=this.prefBranch.getBoolPref("webmail"+o.id+"."+o.user+".enabled");
    }catch(e){}
    if(o.enabled){
      if(o.user==defaults[o.id])this.hdlTable[o.id].splice(0,0,o);//default account inserted into index 0
      else this.hdlTable[o.id].push(o);
      ++this.countEnabled;
    }
  }
  for(var i=0;i<this.hdlTable.length;i++){
    var o=this.hdlTable[i];
    if(o.length>=1){
      var prefStr="webmail"+i+".default";
      var t=null;
      try{
        t=this.prefBranch.getCharPref(prefStr);
      }catch(e){}
      if(t==null)this.prefBranch.setCharPref(prefStr,o[0].user);
    }
  }
  if(aCheckNow){
    this.setState(nsIWebMailNotifier.ST_RESET,"",0);
    this.checkMails();
  }
}
WebMailNotifier.prototype.isDefault = function(id,user) {
  return this.hdlTable[id].length==1||(this.hdlTable[id].length>1&&this.hdlTable[id][0].user==user);
}
WebMailNotifier.prototype.isMultiple = function(id) {
  return this.hdlTable[id].length>1;
}
WebMailNotifier.prototype.getAccountsNumber = function() {
  return this.webMailHandlers.length;
}
WebMailNotifier.prototype.getAccountInfo = function(aIndex) {
  var obj=this.webMailHandlers[aIndex];
  return obj.id+"\t"+obj.user+"\t"+obj.password;
}
WebMailNotifier.prototype.isAccountEnabled = function(aIndex) {
  return this.webMailHandlers[aIndex].enabled;
}
WebMailNotifier.prototype.addListener = function(aListener) {
  this.listeners.push(aListener);
  if (!this.timer){//first listener
    this.checkMails();
    this.setTimer();
  }else{
    for each(var i in this.webMailHandlers){
      var count=this.calcCount(i.id,i.user,i.mailCount);
      aListener.onStateChange(i.id,i.user,count);
    }
  }
}
WebMailNotifier.prototype.removeListener = function(aListener){
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]==aListener){
        this.listeners.splice(i,1);
        break;
    }
  }
}
WebMailNotifier.prototype.checkMails = function() {
  this.countChecked=0;
  for each(var o in this.webMailHandlers)o.checkMail();
}
WebMailNotifier.prototype.getHandler = function(aID,aUser) {
  for each(var i in this.webMailHandlers){
    if(i.id==aID&&i.user==aUser)return i;
  }
  return null;
}
WebMailNotifier.prototype.getMailURL = function(aID,aUser) {
  var obj=this.getHandler(aID,aUser);
  if(obj){
    obj.cookieManager.setCookieToBrowser();
    if(this.newMailsOnly){
      this.prefBranch.setIntPref("webmail"+aID+"."+aUser+".count",obj.mailCount);
      this.setState(aID,aUser,obj.mailCount);
    }
    return obj.mailURL;
  }else return null;
}
WebMailNotifier.prototype.calcCount = function(aID,aUser,aCount) {
  if(this.newMailsOnly){
    var prefStr="webmail"+aID+"."+aUser+".count";
    var count=null;
    try{
      count=this.prefBranch.getIntPref(prefStr);
    }catch(e){}
    if(count==null){
      this.prefBranch.setIntPref(prefStr,0);
      count=0;
    }
    if(aCount>=0){
      if(aCount>=count)aCount-=count;
      else{
        this.prefBranch.setIntPref(prefStr,aCount);
        aCount=0;
      }
    }
  }
  return aCount;
}
WebMailNotifier.prototype.setState = function(aID,aUser,aCount) {
  if(aID>=0){
    ++this.countChecked;
    aCount=this.calcCount(aID,aUser,aCount);
    if(this.countChecked==this.countEnabled){
      var total=0;
      for each(o in this.webMailHandlers){
        if(o.enabled&&o.mailCount>0){
          total+=this.calcCount(o.id,o.user,o.mailCount);
        }
      }
      if(total>this.countTotal)this.notification(total);
      this.countTotal=total;
//      this.notification(total);
//dout("all done");
    }
  }
  for(var i=0;i<this.listeners.length;i++){
    if(this.listeners[i]){
      this.listeners[i].onStateChange(aID,aUser,aCount);
    }
  }
}
WebMailNotifier.prototype.notification = function(total) {
  if(this.prefBranch.getBoolPref("mailAlert")){
    if(this.listeners.length>0){
      try{
        //check alert service
        //doesn't supported in Mac OSX
        var obj = Components.classes["@mozilla.org/alerts-service;1"]
                                       .getService(Components.interfaces.nsIAlertsService);
        this.listeners[0].showAlertNotification(total);
      }catch(e){}
    }
  }
  if(this.prefBranch.getBoolPref("mailSound")){
    if (!this.mSound)this.mSound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);

    var soundLocation = this.prefBranch.getBoolPref("customSound")?
              this.prefBranch.getCharPref("soundUrl") : "_moz_mailbeep";
    if (soundLocation.indexOf("file://") == -1){
      this.mSound.playSystemSound(soundLocation);
    }else{
      var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
      this.mSound.play(ioService.newURI(soundLocation, null, null));
    }
  }
/*

  var obj = Components.classes["@mozilla.org/alerts-service;1"]
                                 .getService(Components.interfaces.nsIAlertsService);
  obj.showAlertNotification("chrome://wm-notifier/content/wmn-icon.png",
                                         "WebMail Notifier","You have new mails", true, "", this);
*/
}

/***********************************************************
timer
***********************************************************/
WebMailNotifier.prototype.setTimer = function() {
  if (this.timer) {
    this.timer.cancel();
  } else {
    this.timer = Components.classes["@mozilla.org/timer;1"]
                  .createInstance(Components.interfaces.nsITimer);
  }
  if(this.timerDelay>0){
    try{
      this.timer.initWithCallback(this,
                                  this.timerDelay,
                                  Components.interfaces.TYPE_ONE_SHOT);
    }catch(e){}
  }
}

WebMailNotifier.prototype.notify = function(aTimer) {
  this.checkMails();
  this.setTimer();
}
/***********************************************************
load html
***********************************************************/
WebMailNotifier.prototype.getHtml = function(aHandler,aURL,aPostData) {
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = ioService.newURI(aURL, null, null);
  var channel = ioService.newChannelFromURI(uri);
  var httpChannel = channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  if (aPostData) {
    var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
                        .createInstance(Components.interfaces.nsIStringInputStream);
    uploadStream.setData(aPostData, aPostData.length);
    var uploadChannel = channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);
    httpChannel.requestMethod = "POST";
  }
  aHandler.channel=channel;
  channel.notificationCallbacks = aHandler;
  channel.asyncOpen(aHandler,httpChannel);
}
/***********************************************************
preference observer
***********************************************************/
WebMailNotifier.prototype.observe=function(aSubject, aTopic, aData)
{
  switch(aTopic){
  case "nsPref:changed":
    switch (aData) {
    case "updateInterval":
      this.timerDelay=this.prefBranch.getIntPref(aData)*60000;
      this.setTimer();
      break;
    case "newMailsOnly":
      this.newMailsOnly=this.prefBranch.getBoolPref("newMailsOnly");
      for each(var o in this.webMailHandlers){
        if(o.enabled){
          this.prefBranch.setIntPref("webmail"+o.id+"."+o.user+".count",0);
          this.setState(o.id,o.user,o.mailCount);
        }
      }
      break;
    }
    break;
  case "http-on-modify-request":
    for each(var i in this.webMailHandlers){
      if(i.channel==aSubject){
        i.cookieRequest(aSubject.QueryInterface(Components.interfaces.nsIHttpChannel));
        break;
      }
    }
    break;
  case "http-on-examine-response":
    for each(var i in this.webMailHandlers){
      if(i.channel==aSubject){
        i.cookieResponse(aSubject.QueryInterface(Components.interfaces.nsIHttpChannel));
        break;
      }
    }
    break;
  }
}
/***********************************************************
password manager
***********************************************************/
WebMailNotifier.prototype.getPassword=function(mailid,userid){
  var passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                .getService(Components.interfaces.nsIPasswordManager);
  var e = passwordManager.enumerator;
  var str="chrome://wm-notifier/webmail"+mailid;
  while (e.hasMoreElements()) {
    try {
      var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
      if (pass.host == str&&pass.user==userid) {
         return pass.password; // the password
      }
    }catch(ex) {
    }
  }
  return "";
}

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var WebMailNotifierFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new WebMailNotifier()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var WebMailNotifierModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return WebMailNotifierFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return WebMailNotifierModule; }

/***********************************************************
CookieManager definition
***********************************************************/
function CookieInfo(){
}
CookieInfo.prototype={
  getDomain:function(){
    if(this.domain){
      if(this.domain.charAt(0)!=".")return "."+this.domain;
      else return this.domain;
    }
    return this.uri.host;
  },
  getPath:function(){
    if(this.path)return this.path;
    var path=this.uri.path;
    var i=path.lastIndexOf("/");
    if(i==-1)return path;
    else return path.substring(0,i+1);
  },
  toString:function(){
      var s=this.name+"="+this.value;
      for(var p in this){
        if(typeof this[p] == "function")continue;
        if(p!="name"&&p!="value"&&p!="uri"&&p!="httponly"&&p!="secure"){
          s+="; "+p+"="+this[p];
        }
      }
    return s;
  }
}
function CookieManager(){
  this.cookies=[];
}
CookieManager.prototype={
  addCookies: function(aURI,aCookie){
    aCookie=aCookie.split("\n");
    for each(var o in aCookie){
      var val=new CookieInfo();
      var ar=o.split(";");
      var t=ar[0].match(/(\S+?)=(.*)/);

      val["name"]=t[1];
      val["value"]=t[2];

      for(var i=1;i<ar.length;i++){
        if(ar[i]){
          var t=ar[i].replace( /^\s+/g,"");
          var f=t.match(/(\S+?)=(.*)/);
          if(f)val[f[1].toLowerCase()]=f[2];
          else val[t.toLowerCase()]="";
        }
      }
      val.uri=aURI;
      this._insert(val);
//if(val.getPath()!=val.path)dout(val.getDomain()+" "+val.getPath()+" "+val.domain+" "+val.path);
    }
  },
  _insert: function(val){
    for(var i in this.cookies){
      var o=this.cookies[i];
      if(o.name==val.name&&o.getDomain()==val.getDomain()&&o.getPath()==val.getPath()){
        this.cookies[i]=val;
        return;
      }
    }
    this.cookies.push(val);
  },
  getCookie: function(aURI){
    var str="";
    for each(var o in this.cookies){
      var domain=o.getDomain();
      if(domain.charAt(0)==".")domain=domain.substring(1);
      if(aURI.host.lastIndexOf(domain)==-1)continue;
      if(aURI.path.indexOf(o.getPath())!=0)continue;
      var exp=o.expires;
      if(exp){
        exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
        if(Date.parse(exp)<new Date()){
          continue;
        }
      }
      var s=o.name+"="+o.value;
      if(str)str+="; "+s;
      else str=s;
    }
    return str;
  },
  setCookieToBrowser: function(){
    var cs = Components.classes["@mozilla.org/cookieService;1"]
                  .getService(Components.interfaces.nsICookieService);

    var cm = Components.classes["@mozilla.org/cookiemanager;1"]
              .getService(Components.interfaces.nsICookieManager2);
    for each(var o in this.cookies){
      cm.remove(o.getDomain(),o.name,o.getPath(),false);//can't set some cookies if they already exists.
      cs.setCookieString(o.uri,null,o.toString(), null);
      ////////////////for debug//////////////////////////////////
      if(enableDebug){
        if(!this.findCookie(o)){
          dlog(o.toString()+" "+o.uri.spec);
        }
      }
      ///////////////////////////////////////////////////////////
    }
  },
////////////////for debug//////////////////////////////////
  findCookie: function(o){
      var exp=o.expires;
      if(exp){
        exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
        if(Date.parse(exp)<new Date()){
          return true;
        }
      }
      var domain=o.getDomain();
      var path=o.getPath();
    var obj = Components.classes["@mozilla.org/cookiemanager;1"].
                getService(Components.interfaces.nsICookieManager2);
    var enm = obj.enumerator;
    while(enm.hasMoreElements()){
      var ck = enm.getNext().QueryInterface(Components.interfaces.nsICookie2);
      if(ck.host==domain&&ck.path==path&&ck.name==o.name&&ck.value==o.value)return true;
    }
    return false;
  },
///////////////////////////////////////////////
  clear: function(){
    this.cookies=[];
  }
}
/***********************************************************
WebMailHandler definition
***********************************************************/
function WebMailHandler(webMailNotifier){
  this.webMailNotifier=webMailNotifier;
  this.cookieManager=new CookieManager();
  this.loggedIn=false;
}
WebMailHandler.prototype={
  charSet: null,
  data: "",
  stage: 0,
  initStage: ST_LOGIN,
  mailCount:-1,
  user: null,
  password: null,

  // nsIChannelEventSink
  onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
    if(this.channel==aOldChannel)this.channel = aNewChannel;
  },
  // nsIInterfaceRequestor
  getInterface: function (aIID) {
    try {
      return this.QueryInterface(aIID);
    } catch (e) {
      throw Components.results.NS_NOINTERFACE;
    }
  },
  // nsIProgressEventSink
  onProgress : function (aRequest, aContext, aProgress, aProgressMax) { },
  onStatus : function (aRequest, aContext, aStatus, aStatusArg) { },
  // nsIHttpEventSink
  onRedirect : function (aOldChannel, aNewChannel) { },
  QueryInterface : function(aIID) {
    if (aIID.equals(Components.interfaces.nsISupports) ||
        aIID.equals(Components.interfaces.nsIInterfaceRequestor) ||
        aIID.equals(Components.interfaces.nsIChannelEventSink) ||
        aIID.equals(Components.interfaces.nsIProgressEventSink) ||
        aIID.equals(Components.interfaces.nsIHttpEventSink) ||
        aIID.equals(Components.interfaces.nsIStreamListener)||
        aIID.equals(Components.interfaces.nsIPrompt)|| //not used but required
        aIID.equals(Components.interfaces.nsIAuthPrompt)||
        aIID.equals(Components.interfaces.nsIAuthPromptProvider))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },
  onStartRequest : function (aRequest, aContext) {
    this.data = "";
  },
  onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
	  var is = Components.classes["@mozilla.org/scriptableinputstream;1"]
            .createInstance(Components.interfaces.nsIScriptableInputStream);
  	is.init(aStream);
  	this.data += is.read(aLength);
  },
  onStopRequest : function (aRequest, aContext, aStatus) {
    var charSet=null;
    var httpChannel=this.channel.QueryInterface(Components.interfaces.nsIHttpChannel);
    if(this.charSet)charSet=this.charSet;
    else{
      var fnd=this.data.match(/<meta.+?\".+?charset=(\S+?)\s*?\"\s*?\x2f*>/);
      if(fnd)charSet=fnd[1];
      else{
        try{
          if(httpChannel.contentCharset)charSet=httpChannel.contentCharset;
        }catch(e){}
      }
    }
    if(charSet){
      var uniConv = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
      								.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
      uniConv.charset = charSet;
      try{
        this.data=uniConv.ConvertToUnicode(this.data);
      }catch(e){}
    }
    if(!this.process(aRequest, aContext, aStatus))++this.stage;
  },
  process : function (aRequest, aContext, aStatus){
    switch(this.stage){
    case ST_LOGIN:
      this.webMailNotifier.getHtml(this,this.loginData[LOGIN_URL],this.loginData[LOGIN_POST]);
      break;
    case ST_LOGIN_RES:
      this.stage=ST_DATA;
    case ST_DATA:
      this.webMailNotifier.getHtml(this,this.dataURL);
      break;
    case ST_DATA_RES:
      this.mailCount=this.getData(this.data);
      this.stage=this.mailCount<0?this.initStage:ST_DATA;
      this.webMailNotifier.setState(this.id,this.user,this.mailCount);
      if(this.mailCount>=0&&!this.loggedIn&&this.webMailNotifier.isDefault(this.id,this.user)){
        this.loggedIn=true;
        this.cookieManager.setCookieToBrowser();
      }
      return true;
    }
  },
  checkMail : function(){
    if(!this.enabled)return;
    if(!this.process(null,null,0))++this.stage;
  },
  cookieRequest : function(aHttpChannel){
    if(this.stage==this.initStage){
      aHttpChannel.setRequestHeader("Cookie", "", false);//try clean login
      this.cookieManager.clear();
    }else{
      var ck=this.cookieManager.getCookie(aHttpChannel.URI);
      aHttpChannel.setRequestHeader("Cookie",ck, false);
    }
  },
  cookieResponse : function(aHttpChannel){
    try {
      var cookie = aHttpChannel.getResponseHeader("Set-Cookie");
      this.cookieManager.addCookies(aHttpChannel.URI,cookie);
      aHttpChannel.setResponseHeader("Set-Cookie", "", false);
    }catch(e){}
  }
};

////////////////for debug//////////////////////////////////
function dlog(str){
  dout(str);
  var name="wmn-log.txt";
  var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
  file.append(name);
  try{
    var fos = Components.classes["@mozilla.org/network/file-output-stream;1"]
                          .createInstance(Components.interfaces.nsIFileOutputStream);
    if(enableDebug==2)this.notFirst=true;
    if(this.notFirst){
      fos.init(file, 0x02 | 0x08 | 0x10 , 0664, 0);
    }else{
      this.notFirst=true;
      fos.init(file, 0x02 | 0x08 | 0x20, 0664, 0);
    }
    var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
                       .createInstance(Components.interfaces.nsIConverterOutputStream);
    os.init(fos, "UTF-8", 0, 0x0000);
    str=str.replace(/\r/g,"\\r").replace(/\n/g,"\\n")+"\r\n";
    os.writeString(new Date()+"\t");
    os.writeString(str);
    os.close();
    fos.close();
  }catch(e){
dout(e);
  }
}
///////////////////////////////////////////////////////////
